

' ========================================================================================
' Find/Replace class to wrap the common dialog FindReplace
' ========================================================================================
type clsFindReplace
   private:
      _hDialog         AS HWND             
      _fr              AS FINDREPLACE      
      _RegMsg          AS UINT             
      _wszFindText     AS WSTRING * MAX_PATH
      _wszReplaceText  AS WSTRING * MAX_PATH
      _wszLastFound    AS WSTRING * MAX_PATH

   public:
      declare function ShowDialog( BYVAL hParent AS HWND, byval hRichEdit as HWND, byval bShowFind as Boolean ) AS UINT
      declare function FindNext( byval hRichEdit as HWND, byval nFlags as long ) as boolean
      declare function FindReplaceAction( byval hRichEdit as HWND ) as Long
      declare function hDialog() as HWND
end type

dim shared cFindReplace as clsFindReplace


''
''
function clsFindReplace.hDialog() as HWND
   function = _hDialog
end function


''
''
function clsFindReplace.ShowDialog( BYVAL hParent AS HWND, _
                                    byval hRichEdit as HWND, _
                                    byval bShowFind as Boolean _
                                    ) AS UINT

   ' If the FindReplace dialog already exists then exit
   if _hDialog THEN exit function

   ' Prefill the find textbox with selected text.
   dim as string buffer = RichEdit_GetSelText( hRichEdit )
   IF LEN(buffer) THEN
      _wszFindText = buffer
      _wszLastFound = buffer
   ELSE
      _wszFindText = _wszLastFound
   END IF

   ' Register a Windows message to communicate with the dialog
   _RegMsg = RegisterWindowMessage("commdlg_FindReplace")

   _fr.lStructSize      = SIZEOF(_fr)
   _fr.hwndOwner        = hParent
   _fr.hInstance        = NULL
   _fr.Flags            = FR_DOWN
   _fr.lpstrFindWhat    = @_wszFindText
   _fr.lpstrReplaceWith = @_wszReplaceText
   _fr.wFindWhatLen     = MAX_PATH
   _fr.wReplaceWithLen  = MAX_PATH
   _fr.lCustData        = 0
   _fr.lpfnHook         = nULL
   _fr.lpTemplateName   = nULL

   IF bShowFind THEN
      _hDialog = FindText(@_fr)
   ELSE
      _hDialog = ReplaceText(@_fr)
   END IF
   
   function = _RegMsg

END function


''
''
function clsFindReplace.FindNext( byval hRichEdit as HWND, byval nFlags as long ) as boolean

   dim as string buffer
   dim as long startPos, endPos
   
   buffer = RichEdit_GetSelText( hRichEdit )
   IF LEN(buffer) THEN
      _wszFindText = buffer
      _wszLastFound = buffer
   end if   
   RichEdit_GetSel( hRichEdit, @startPos, @endPos )

   dim ftexw as FINDTEXTEXW
   if (nFlags and FR_DOWN) = FR_DOWN then
      ftexw.chrg.cpMin = endPos + 1
      ftexw.chrg.cpMax = RichEdit_GetTextLength( hRichEdit )
   else
      ftexw.chrg.cpMin = startPos - 1
      ftexw.chrg.cpMax = 0
   end if      
   ftexw.lpstrText = @_wszFindText
   
   if RichEdit_FindTextEx( hRichEdit, nFlags, @ftexw ) <> -1 then  
      RichEdit_SetSel( hRichEdit, ftexw.chrgText.cpMin, ftexw.chrgText.cpMax )
      return true
   end if
   
   function = false
END function


''
''
function clsFindReplace.FindReplaceAction( BYVAL hRichEdit AS HWND ) as Long

   IF (_fr.flags AND FR_DIALOGTERM) = FR_DIALOGTERM THEN
      IF _wszLastFound <> _wszFindText THEN _wszLastFound = _wszFindText
      _hDialog = 0
      exit function
   End if
         
   IF (_fr.flags AND FR_FINDNEXT) = FR_FINDNEXT THEN
      ' Find the next match
      this.FindNext( hRichEdit, _fr.Flags )
      
   ELSEIF (_fr.flags AND FR_REPLACE) = FR_REPLACE THEN
      ' If no current selection then search for first occurance now
      if len(RichEdit_GetSelText(hRichEdit)) = 0 then
         _fr.Flags = _fr.Flags OR FR_FINDNEXT
         this.FindNext( hRichEdit, _fr.Flags ) 
         exit function
      end if
         
      ' Replace the current selected text
      RichEdit_ReplaceSel( hRichEdit, true, @_wszReplaceText )

      ' Find the next match
      _fr.Flags = _fr.Flags OR FR_FINDNEXT
      this.FindNext( hRichEdit, _fr.Flags )

   ELSEIF (_fr.flags AND FR_REPLACEALL) = FR_REPLACEALL THEN
      ' Turn off window redrawing in order to avoid flicker. 
      SendMessage( hRichEdit, WM_SETREDRAW, false, 0 )

      ' We need to position to the start of the edit buffer in order
      ' to start our searching of the entire document.
      RichEdit_SetSel( hRichEdit, 0, 0 )

      ' Do the replacements until no text is found.
      _fr.Flags = _fr.Flags OR FR_FINDNEXT
      do until this.FindNext( hRichEdit, _fr.Flags ) = false
         RichEdit_ReplaceSel( hRichEdit, true, @_wszReplaceText )
      loop

      ' Re-enable control drawing
      SendMessage( hRichEdit, WM_SETREDRAW, true, 0 )
      AfxRedrawWindow( hRichEdit )
      
   END IF

   function = 0
END function

