
''
''
function EditorPrintDialog() as Boolean
   
   dim cp as CPRINT
   
   function = cp.ChoosePrinter( frmMain.hWindow )

   AfxMsg( "Actual printing code not implemented." )
   
end function

''
''
function EditorPageSetupDialog() as Boolean
   
   dim cp as CPRINT
   
   function = cp.PageSetup( frmMain.hWindow )

end function

''
''
function EditorFontDialog() as Boolean

   dim hFont as HFONT
   dim cf as CHOOSEFONT 
   dim lf as LOGFONT
   
   hFont = AfxGetWindowFont(frmMain.txtEdit.hWindow)
   GetObject( hFont, sizeof(lf), @lf )
   
   cf.lStructSize = sizeof(cf)
   cf.hwndOwner   = frmMain.hWindow
   cf.lpLogFont   = @lf
   cf.Flags       = CF_SCREENFONTS or CF_EFFECTS or CF_INITTOLOGFONTSTRUCT

   if ChooseFont(@cf) then
      ' Delete the current font and assign a newly created font
      DeleteObject(hFont)
      hFont = AfxCreateFont( cf.lpLogFont->lfFaceName, _
                             cf.lpLogFont->lfHeight, _
                             -1, _                        ' -1 to use system DPI settings
                             cf.lpLogFont->lfWeight, _
                             cf.lpLogFont->lfItalic, _
                             cf.lpLogFont->lfUnderline, _
                             cf.lpLogFont->lfStrikeOut, _
                             cf.lpLogFont->lfCharSet _
                             )
      AfxSetWindowFont(frmMain.txtEdit.hWindow, hFont, true) 
      return true
   END IF

   function = false
end function

''
''
function EditorEditMenuStates() as Long
   ' Set the states of the menu items in the Edit menu.
   dim as Boolean bNoCutCopy, bNoUndo, bNoPaste
   
   dim as HWND hEdit = frmMain.txtEdit.hWindow
   
   bNoCutCopy = iif( frmMain.txtEdit.SelectionLength = 0, true, false)
   bNoUndo    = iif( Edit_CanUndo(hEdit), false, true )
   bNoPaste   = iif( len(AfxGetClipboardText) = 0, true, false )
   
   frmMain.MainMenu.ByMenuName("mnuUndo").Grayed   = bNoUndo
   frmMain.MainMenu.ByMenuName("mnuCut").Grayed    = bNoCutCopy
   frmMain.MainMenu.ByMenuName("mnuCopy").Grayed   = bNoCutCopy
   frmMain.MainMenu.ByMenuName("mnuPaste").Grayed  = bNoPaste
   frmMain.MainMenu.ByMenuName("mnuDelete").Grayed = bNoCutCopy
   
   function = 0
end function

''
''
function EditorUpdateStatusBar() as Long
   ' Display it in the status bar
   frmMain.StatusBar.Panel(1).Text = _
      "Ln "    & frmMain.txtEdit.LinePosition + 1 & _
      ", Col " & frmMain.txtEdit.ColumnPosition + 1
   function = 0
end function

''
''
function EditorUpdateCaption() as Long
   
   ' Display the filename in the form title bar
   dim as CWSTR wszFilename 
   if gDoc.IsNew then 
      wszFilename = "Untitled"
   else   
      wszFilename = AfxStrPathName( "NAMEX", gDoc.wszFilename ) 
   end if
   if Edit_GetModify( frmMain.txtEdit.hWindow ) then 
      wszFilename = "*" & wszFilename
   end if   
   
   ' Only update if different in order to avoid flicker
   if frmMain.Text <> wszFilename then
      frmMain.Text = wszFilename & " - Notepad"
   end if
   
   function = 0
end function

''
''
function EditorSaveFileAs() as boolean

   DIM as CWSTR wszFilename, wszInitialDir, wszFilter, wszResult 
   dim as DWORD dwFlags
    
   wszFilter = "Text Documents (*.txt)|*.txt|" & "All Files (*.*)|*.*|"
   dwFlags   = OFN_EXPLORER OR OFN_FILEMUSTEXIST OR OFN_HIDEREADONLY or OFN_OVERWRITEPROMPT
    
   if gDoc.IsNew then
      wszFilename = "Untitled"
      wszInitialDir = CURDIR
   else
      wszFilename = gDoc.wszFilename 
      wszInitialDir = AfxStrPathName("PATH", gDoc.wszFilename) 
   end if
   
   wszResult = AfxSaveFileDialog( frmMain.hWindow, "Save As", wszFilename, _
                                  wszInitialDir, wszFilter, _
                                  "txt", @dwFlags )
   if len(wszResult) then
      gDoc.wszFilename = wszResult 
      function = true  ' success
   else
      function = false  ' save was canceled
   end if
end function

''
''
function EditorSaveFile( byval bSaveAs as Boolean ) as boolean
   ' Save any currently open file if modifications are pending
   if Edit_GetModify( frmMain.txtEdit.hWindow ) = false then return true
   
   if (gDoc.IsNew = true) or (bSaveAs = true) then
      ' If SaveAs returns False then the dialog was cancelled
      if EditorSaveFileAs() = false then return false
   end if   
   dim cText as CTextStream 
   cText.OpenForOutputW( gDoc.wszFilename, true )
   cText.Write frmMain.txtEdit.Text
   cText.Close
   Edit_SetModify( frmMain.txtEdit.hWindow, false )
   EditorUpdateCaption()
   function = true   ' success
end function


''
''
function EditorOpenFile( byval wszFilename as CWSTR ) as boolean
   
   dim nResult as Long = false  ' default to failure
   
   ' Save any currently open document
   ' If False, then we aborted out of a SaveAs via selecting Cancel
   if EditorSaveFile() = false then return false

   if AfxFileExists( wszFilename ) then
      gDoc.IsNew = false
      ' Load the document and set state
      gDoc.wszFilename = wszFilename
      gDoc.IsUnicode = AfxIsFileUnicode( gDoc.wszFilename )
      dim cText as CTextStream 
      cText.Open( gDoc.wszFilename, IOMODE_ForReading, false, gDoc.IsUnicode )
      frmMain.txtEdit.Text = cText.ReadAll()
      cText.Close
      nResult = true  ' success
   else
      gDoc.IsNew = true
      frmMain.txtEdit.Text = ""
      nResult = false  ' failure (new document created instead)
   end if
   
   ' Ensure focus is set back to the edit window
   frmMain.txtEdit.Focused = true
   
   EditorUpdateCaption() 
   EditorUpdateStatusBar()
   
   function = nResult
end function

''
''
function EditorOpenDialog() as boolean
   ' Save any currently open document
   ' If False, then we aborted out of a SaveAs via selecting Cancel
   if EditorSaveFile() = false then return false

   DIM as CWSTR wszFilename, wszInitialDir, wszFilter, wszResult 
   dim as DWORD dwFlags
   
   wszInitialDir = CURDIR
   wszFilter     = "Text Documents (*.txt)|*.txt|" & "All Files (*.*)|*.*|"
   dwFlags       = OFN_EXPLORER OR OFN_FILEMUSTEXIST OR OFN_HIDEREADONLY
   
   wszResult = AfxOpenFileDialog( frmMain.hWindow, "Open", "", _
                                  wszInitialDir, wszFilter, _
                                  "txt", @dwFlags, NULL)
   if len(wszResult) then
      function = EditorOpenFile( wszResult )
   else
      ' The open dialog was cancelled
      function = false   
   end if
   
end function

